//------------------------------------------------------------------------------
//
//  npdata.cpp
//  npe
//
//  Created by Shane Saxon on 2/19/10.
//  Copyright Saxon Digital 2010. All rights reserved.
//
//------------------------------------------------------------------------------


#include "npdata.h"
#include "npctrl.h"
#include "data/npplot.h"
#include "io/npkey.h"

#include <stdlib.h>

void* gData;

void npInitDataMap (void* dataRef);
void npInitDataIO (void* dataRef);
void npInitDataCtrl (void* dataRef);


//------------------------------------------------------------------------------
void npInitDataMap(void* dataRef)
{
	pData data = (pData) dataRef;

	NPplotPtr plot			= NULL;
	NPcameraPtr cameraData	= NULL;
	NPpointsPtr pointsData	= NULL;
	NPpinPtr pinData		= NULL;
	NPvideoPtr videoData	= NULL;
	NPgridPtr gridData		= NULL;

	data->map.plotCount		= 0;
	data->map.plotIndex		= 0;
	npInitPlot (dataRef);

	data->map.activePlot	= NULL;
	data->map.selectedPlot	= NULL;
	data->map.selectedIndex = 0;

	data->map.selectionSet.x = false;
	data->map.selectionSet.y = false;
	data->map.selectionSet.z = false;
	data->map.selectAll		= false;

	npPlotNew (kPlotDefault, dataRef);			//global parent at index = 0

	// setup our primary camera
	plot = npPlotNew (kPlotCamera, dataRef);	//camera at index = 1
	npDataCameraReset (plot, data);

	// setup objects
	plot = npPlotNew (kPlotGrid, dataRef);

	plot->scale.z = 1.5f;



//	npDataPreset (1, dataRef);
	npCtrlCommand (kGraphCamera, dataRef);
}


//------------------------------------------------------------------------------
void npInitDataIO(void* dataRef)
{
	int i = 0;

	pData data = (pData) dataRef;

	npInitKey (dataRef);

	data->io.channelCount = 1;			// the first channel is the sample time

	data->io.drawMenu = true;
	data->io.clearFrameBuffer = true;

	data->io.axes.x = true;
	data->io.axes.y = true;
	data->io.axes.z = true;

	data->io.camTranslate[0].x = 0.0f;
	data->io.camTranslate[0].y = 0.0f;
	data->io.camTranslate[0].z = 0.0f;
	data->io.camRotate[0].x = 0.0f;
	data->io.camRotate[0].y = 0.0f;
	data->io.camRotate[0].z = 0.0f;

	data->io.camTranslate[1].x = 0.0f;
	data->io.camTranslate[1].y = 0.0f;
	data->io.camTranslate[1].z = 100.0f;
	data->io.camRotate[1].x = kPos2PI / 4.0f;
	data->io.camRotate[1].y = 0.0f;
	data->io.camRotate[1].z = 0.0f;

	data->io.camTranslate[2].x = 0.0f;
	data->io.camTranslate[2].y = -85.0f;
	data->io.camTranslate[2].z = 7.0f;
	data->io.camRotate[2].x = 0.0f;
	data->io.camRotate[2].y = 0.0f;
	data->io.camRotate[2].z = 0.0f;

	data->io.camTranslate[3].x = 85.0f;
	data->io.camTranslate[3].y = 0.0f;
	data->io.camTranslate[3].z = 7.0f;
	data->io.camRotate[3].x = 0.0f;
	data->io.camRotate[3].y = 0.0f;
	data->io.camRotate[3].z = -kPos2PI / 4.0f;


	data->io.camIndex = 0;

	data->io.size = 0;
}


//------------------------------------------------------------------------------
void npInitDataCtrl(void* dataRef)
{
	
}


//------------------------------------------------------------------------------
void npInitData (int argc, char* argv[], void* dataRef)
{
	pData data = (pData) malloc (sizeof(Data));
	
	npInitDataMap (data);
	npInitDataIO (data);
	npInitDataCtrl (data);

	dataRef = gData = data;

	// create a fixed array of global data pointers that is thread safe,		ZZ
}


//------------------------------------------------------------------------------
void npCloseData()
{
	pData data = (pData) gData;
	
	free (data);
}


//------------------------------------------------------------------------------
void* npDataGetDataRef()
{
	return gData;
}


//------------------------------------------------------------------------------
void npDataUpdate (void* dataRef)
{

}

// loads preset scene using currently selected object as the origin
// if alt is pressed (ie ALT+5) it loads preset at the global origin
//------------------------------------------------------------------------------
void npDataPreset (int preset, void* dataRef)
{
	int i = 0;
	int j = 0;

	GLfloatXYZ	translateOffset;
	GLfloatXYZ	rotateOffset;
	GLfloatXYZ	scaleOffset;

	pData data = (pData) dataRef;
	NPplotPtr plot = data->map.plot[data->map.plotIndex];

	NPplotPtr plotChild = NULL;

	NPpinPtr pinData = NULL;

	// don't offset if alt key held or camera selected
	if (data->io.key.modAlt || data->map.plotIndex == 1)
	{
		translateOffset.x = 0.0f;
		translateOffset.y = 0.0f;
		translateOffset.z = 0.0f;

		rotateOffset.x = 0.0f;
		rotateOffset.y = 0.0f;
		rotateOffset.z = 0.0f;

		scaleOffset.x = 0.0f;
		scaleOffset.y = 0.0f;
		scaleOffset.z = 0.0f;
	}
	else
	{
					// add code to calc translate, rotate position based on
					// global coordinates think of GL order of operation,	debug zz
		translateOffset.x = plot->translate.x;
		translateOffset.y = plot->translate.y;
		translateOffset.z = plot->translate.z;

		rotateOffset.x = plot->rotateRad.x;
		rotateOffset.y = plot->rotateRad.y;
		rotateOffset.z = plot->rotateRad.z;

		scaleOffset.x = plot->scale.x;
		scaleOffset.y = plot->scale.y;
		scaleOffset.z = plot->scale.z;
	}

	switch (preset)
	{
		case kGraphPresetOne :
					plot = npPlotNew (kPlotPin, dataRef);
						plot->colorIndex = 15;
					plotChild = plot->child[0];
						plotChild->colorIndex = 14;

					plotChild = npPlotNewChild (plot, dataRef);
						plotChild->colorIndex = 1;
					plotChild = npPlotNewChild (plot, dataRef);
						plotChild->colorIndex = 14;
					plotChild = npPlotNewChild (plot, dataRef);
						plotChild->colorIndex = 1;
					plot = npPlotNewChild (plot, dataRef);
						plot->colorIndex = 2;
					plotChild = npPlotNewChild (plot, dataRef);
						plotChild->colorIndex = 1;
					plot = npPlotNewChild (plot, dataRef);
						plot->colorIndex = 3;
					plotChild = npPlotNewChild (plot, dataRef);
						plotChild->colorIndex = 4;

					npCtrlCommand (kGraphSelectAll, data);
					npCtrlCommand (kGraphColorUp, data);
					npCtrlCommand (kGraphColorDown, data);
					npCtrlCommand (kGraphSelectAll, data);

					data->map.plotIndex = data->map.plotCount - 1;
					data->map.selectedIndex = data->map.plotIndex;
					data->map.selectedPlot = data->map.plot[data->map.selectedIndex];
					data->map.activePlot = data->map.selectedPlot;
			break;

		case kGraphPresetTwo :

			// draw 6x6 grid of pins
			for(i = -3; i < 3; i++)
				for(j=-3; j < 3; j++)
				{
					// create the pin trunk
					plot = npPlotNew (kPlotPin, dataRef);
					plot->translate.x = i * 10.0f;
					plot->translate.y = j * 10.0f;

					plot->translate.x += translateOffset.x;
					plot->translate.y += translateOffset.y;
					plot->translate.z += translateOffset.z;

					// create child plot toroids...
				//	plot = plot->child[plot->childIndex];
/*					npPlotNewChild (plot, dataRef);
					npPlotNewChild (plot, dataRef);
					npPlotNewChild (plot, dataRef);
					plot = npPlotNewChild (plot, dataRef);
					npPlotNewChild (plot, dataRef);
					plot = npPlotNewChild (plot, dataRef);
					npPlotNewChild (plot, dataRef);
					*/
		//			plot = npPlotNew (kPlotPin, dataRef);
						plot->colorIndex = 15;
					plotChild = plot->child[0];
						plotChild->colorIndex = 14;

					plotChild = npPlotNewChild (plot, dataRef);
						plotChild->colorIndex = 1;
					plotChild = npPlotNewChild (plot, dataRef);
						plotChild->colorIndex = 14;
					plotChild = npPlotNewChild (plot, dataRef);
						plotChild->colorIndex = 1;
					plot = npPlotNewChild (plot, dataRef);
						plot->colorIndex = 2;
					plotChild = npPlotNewChild (plot, dataRef);
						plotChild->colorIndex = 1;
					plot = npPlotNewChild (plot, dataRef);
						plot->colorIndex = 3;
					plotChild = npPlotNewChild (plot, dataRef);
						plotChild->colorIndex = 4;
					data->map.plotIndex = data->map.plotCount - 1;
					data->map.selectedIndex = data->map.plotIndex;
					data->map.selectedPlot = data->map.plot[data->map.selectedIndex];
					data->map.activePlot = data->map.selectedPlot;


				}			
			break;
		case kGraphPresetThree :
plot = npPlotNew (kPlotPin, dataRef);
											plot->rotateRate.z = 0.001f;
						plot->colorIndex = 15;
					plotChild = plot->child[0];
						plotChild->colorIndex = 14;

					plotChild = npPlotNewChild (plot, dataRef);
						plotChild->colorIndex = 1;
					plotChild = npPlotNewChild (plot, dataRef);
						plotChild->colorIndex = 14;
					plotChild = npPlotNewChild (plot, dataRef);
						plotChild->colorIndex = 1;
					plot = npPlotNewChild (plot, dataRef);
						plot->colorIndex = 2;
					plotChild = npPlotNewChild (plot, dataRef);
						plotChild->colorIndex = 1;
					plot = npPlotNewChild (plot, dataRef);
						plot->colorIndex = 3;
					plotChild = npPlotNewChild (plot, dataRef);
						plotChild->colorIndex = 4;

plot = npPlotNew (kPlotPin, dataRef);
											plot->rotateRate.z = -0.001f;
						plot->colorIndex = 15;
					plotChild = plot->child[0];
						plotChild->colorIndex = 14;

					plotChild = npPlotNewChild (plot, dataRef);
						plotChild->colorIndex = 1;
					plotChild = npPlotNewChild (plot, dataRef);
						plotChild->colorIndex = 14;
					plotChild = npPlotNewChild (plot, dataRef);
						plotChild->colorIndex = 1;
					plot = npPlotNewChild (plot, dataRef);
						plot->colorIndex = 2;
					plotChild = npPlotNewChild (plot, dataRef);
						plotChild->colorIndex = 1;
					plot = npPlotNewChild (plot, dataRef);
						plot->colorIndex = 3;
					plotChild = npPlotNewChild (plot, dataRef);
						plotChild->colorIndex = 4;

plot = npPlotNew (kPlotPin, dataRef);
						plot->colorIndex = 15;
					plotChild = plot->child[0];
						plotChild->colorIndex = 15;

					plotChild = npPlotNewChild (plot, dataRef);
						plotChild->colorIndex = 1;
					plotChild = npPlotNewChild (plot, dataRef);
						plotChild->colorIndex = 14;
					plotChild = npPlotNewChild (plot, dataRef);
						plotChild->colorIndex = 1;
					plot = npPlotNewChild (plot, dataRef);
										plot->rotateRate.x = 0.001f;
						plot->colorIndex = 2;
					plotChild = npPlotNewChild (plot, dataRef);
						plotChild->colorIndex = 1;
					plot = npPlotNewChild (plot, dataRef);
											plot->rotateRate.x = 0.005f;
						plot->colorIndex = 3;
					plotChild = npPlotNewChild (plot, dataRef);
						plotChild->colorIndex = 4;

plot = npPlotNew (kPlotPin, dataRef);
											plot->rotateRate.z = 0.005f;
						plot->colorIndex = 15;
					plotChild = plot->child[0];
						plotChild->colorIndex = 14;

					plotChild = npPlotNewChild (plot, dataRef);
						plotChild->colorIndex = 1;
					plotChild = npPlotNewChild (plot, dataRef);
						plotChild->colorIndex = 14;
					plotChild = npPlotNewChild (plot, dataRef);
						plotChild->colorIndex = 1;
					plot = npPlotNewChild (plot, dataRef);
						plot->colorIndex = 2;
					plotChild = npPlotNewChild (plot, dataRef);
						plotChild->colorIndex = 2;
					plot = npPlotNewChild (plot, dataRef);
						plot->colorIndex = 3;
					plotChild = npPlotNewChild (plot, dataRef);
						plotChild->colorIndex = 4;

plot = npPlotNew (kPlotPin, dataRef);
											plot->rotateRate.z = 0.001f;
						plot->colorIndex = 15;
					plotChild = plot->child[0];
						plotChild->colorIndex = 14;

					plotChild = npPlotNewChild (plot, dataRef);
						plotChild->colorIndex = 1;
					plotChild = npPlotNewChild (plot, dataRef);
						plotChild->colorIndex = 14;
					plotChild = npPlotNewChild (plot, dataRef);
						plotChild->colorIndex = 1;
					plot = npPlotNewChild (plot, dataRef);
						plot->colorIndex = 2;
					plotChild = npPlotNewChild (plot, dataRef);
						plotChild->colorIndex = 1;
					plot = npPlotNewChild (plot, dataRef);
						plot->colorIndex = 3;
					plotChild = npPlotNewChild (plot, dataRef);
						plotChild->colorIndex = 4;

					npCtrlCommand (kGraphSelectAll, data);
					npCtrlCommand (kGraphColorUp, data);
					npCtrlCommand (kGraphColorDown, data);
					npCtrlCommand (kGraphSelectAll, data);

						
					data->map.plotIndex = data->map.plotCount - 1;
					data->map.selectedIndex = data->map.plotIndex;
					data->map.selectedPlot = data->map.plot[data->map.selectedIndex];
					data->map.activePlot = data->map.selectedPlot;

					npCtrlCommand (kGraphNext, data);
					npCtrlCommand (kGraphNext, data);
					npCtrlCommand (kGraphNext, data);
					npCtrlCommand (kGraphNext, data);
	
			break;
		case kGraphPresetFour : 
					plot = npPlotNew (kPlotPin, dataRef);
										plot->rotateRad.z = 1.7f;
										plot->scale.x = 1.5f;
										plot->scale.y = 1.5f;
										plot->scale.z = 1.5f;

										plot->translate.x -= 3.0f;
										plot->translate.y -= 22.0f;

						plot->colorIndex = 15;
					plotChild = plot->child[0];
						plotChild->colorIndex = 14;
/*
					plotChild = npPlotNewChild (plot, dataRef);
						plotChild->colorIndex = 1;
	*/			plotChild = npPlotNewChild (plot, dataRef);
						plotChild->colorIndex = 15;
					plotChild = npPlotNewChild (plot, dataRef);
						plotChild->colorIndex = 15;
					plot = npPlotNewChild (plot, dataRef);
						plot->colorIndex = 1;
					plotChild = npPlotNewChild (plot, dataRef);
						plotChild->colorIndex = 14;
					plot = npPlotNewChild (plot, dataRef);
						plot->colorIndex = 4;
					plot = npPlotNewChild (plot, dataRef);
						plot->colorIndex = 3;
										plot->rotateRad.x = -0.7f;
					plot = npPlotNewChild (plot, dataRef);
						plot->colorIndex = 3;
										plot->rotateRad.x = 0.7f;
					plot = npPlotNewChild (plot, dataRef);
						plot->colorIndex = 4;
										plot->rotateRad.x = 0.7f;
					plot = npPlotNewChild (plot, dataRef);
						plot->colorIndex = 2;
										plot->rotateRad.x = 0.7f;
					plot = npPlotNewChild (plot, dataRef);
						plot->colorIndex = 1;
										plot->rotateRad.x = 1.5f;


					npCtrlCommand (kGraphSelectAll, data);
					npCtrlCommand (kGraphColorUp, data);
					npCtrlCommand (kGraphColorDown, data);
					npCtrlCommand (kGraphSelectAll, data);

					data->map.plotIndex = data->map.plotCount - 1;
					data->map.selectedIndex = data->map.plotIndex;
					data->map.selectedPlot = data->map.plot[data->map.selectedIndex];
					data->map.activePlot = data->map.selectedPlot;
			break;

		case kGraphPresetFive : break;
		case kGraphPresetSix : break;
		case kGraphPresetSeven : break;
		case kGraphPresetEight : break;

		default : break;
	}

//	data->map.plotIndex = 1;			// select the camera
}


//------------------------------------------------------------------------------
void npDataCameraReset (NPplotPtr plot, void* dataRef)
{
	npInitPlotDefault (plot);
	npInitPlotCamera (plot);

	plot->translate.x = -27.0f;
	plot->translate.y = -25.0f;
	plot->translate.z = 9.0f;
	
	plot->rotate.angle = 0.0f;
	plot->rotate.x = 0.0f;
	plot->rotate.y = 1.0f;
	plot->rotate.z = 0.0f;

	plot->rotateRad.x = kPos2PI / 32.0f;
	plot->rotateRad.z = kPos2PI / 12.0f;

//	plot->triggerOnLow.z = true;				//prevents going below surface
//	plot->triggerLevelLow.z = 1.5f;
}

//------------------------------------------------------------------------------
void npDataCameraResetTwo (NPplotPtr plot, void* dataRef)
{
	npInitPlotDefault (plot);
	npInitPlotCamera (plot);

	plot->translate.x = -0.0f;
	plot->translate.y = -0.0f;
	plot->translate.z = 100.0f;
	
	plot->rotate.angle = 0.0f;
	plot->rotate.x = 0.0f;
	plot->rotate.y = 1.0f;
	plot->rotate.z = 0.0f;

	plot->rotateRad.x = kPos2PI / 4.0f;

//	plot->triggerOnLow.z = true;				//prevents going below surface
//	plot->triggerLevelLow.z = 1.5f;
}

//------------------------------------------------------------------------------
void npDataCameraResetThree (NPplotPtr plot, void* dataRef)
{
	npInitPlotDefault (plot);
	npInitPlotCamera (plot);

	plot->translate.x = -0.0f;
	plot->translate.y = -85.0f;
	plot->translate.z = 7.0f;
	
	plot->rotate.angle = 0.0f;
	plot->rotate.x = 0.0f;
	plot->rotate.y = 1.0f;
	plot->rotate.z = 0.0f;

//	plot->triggerOnLow.z = true;				//prevents going below surface
//	plot->triggerLevelLow.z = 1.5f;
}

//------------------------------------------------------------------------------
void npDataCameraResetFour (NPplotPtr plot, void* dataRef)
{
	npInitPlotDefault (plot);
	npInitPlotCamera (plot);

	plot->translate.x = 85.0f;
	plot->translate.y = 0.0f;
	plot->translate.z = 7.0f;
	
	plot->rotate.angle = 0.0f;
	plot->rotate.x = 0.0f;
	plot->rotate.y = 1.0f;
	plot->rotate.z = 0.0f;

	plot->rotateRad.z = -kPos2PI / 4.0f;

//	plot->triggerOnLow.z = true;				//prevents going below surface
//	plot->triggerLevelLow.z = 1.5f;
}